import bpy
import Mathutils
import math
import os
from bpy.props import *

is_editmode=False
active_ob=None

def create_working_object(ob):

    ob.selected=False

    me=ob.create_mesh(apply_modifiers=True,settings="RENDER")
    me.update()
    obnew=bpy.data.add_object('MESH',"Test")
    obnew.data=me

    bpy.context.scene.objects.link(obnew)
    obnew.selected=True
    bpy.context.scene.objects.active=obnew

    set_sel=[(1) for i in range(len(me.verts))]
    me.verts.foreach_set("selected",set_sel)

    bpy.ops.object.mode_set(mode='EDIT',toggle=True)
    bpy.ops.mesh.quads_convert_to_tris()
    bpy.ops.object.mode_set(mode='OBJECT',toggle=False)
    return obnew

def delete_working_object(obw):
    me=obw.data
    bpy.ops.object.mode_set(mode='OBJECT',toggle=False)
    obw.selected=True
    bpy.context.scene.objects.active=obw
    bpy.ops.object.delete()
    bpy.data.remove_mesh(me)



def save_context():
    global is_editmode,active_ob

    active_ob=bpy.context.active_object

    is_editmode = (active_ob.mode=='EDIT')
    if (is_editmode):
        bpy.ops.object.mode_set(mode='OBJECT',toggle=False)

    return

def restore_context():
    global is_editmode,active_ob

    if (bpy.context.scene.objects.active!=None):
        bpy.ops.object.mode_set(mode='OBJECT',toggle=False)
    active_ob.selected=True
    bpy.context.scene.objects.active=active_ob

    if (is_editmode):
        bpy.ops.object.mode_set(mode='EDIT',toggle=True)

    return

def create_file(fn):

    fn=fn.split('.')
    fnn=fn[0]
    if (len(fn)>2):
        for i in range(1,len(fn)-1):
            fnn=fnn+'.'+fn[i]
    fnn=fnn+'.tri'
    print ("File Name %s" % fnn)
    f=open(fnn,'w')
    f.write ('1\n')
    return f

def export(me,file):

    li=0
    vxyz=[[],[],[]]
    vnxyz=[[],[],[]]
    n_vertex_color_layer=len(me.vertex_colors)
    if (n_vertex_color_layer>1):
        print ("Found %d vertex color layer, using only active one." % n_vertex_color_layer)

    n_uvtex_layer=len(me.uv_textures)
    print ("Found %d UV Tex layer. Using UV Texture order for export." % n_uvtex_layer)

    for face in me.faces:
        uv=[]
        uv_im=[]
        uv_tex_flag=0;
        vert_col=[]
        vert_col_flag=0

        #Collect face texture if any
        for uvtex in me.uv_textures:
            if (uvtex.data[face.index].tex==True):
                if (uvtex.data[face.index].image.name):
                    uv.append(uvtex.data[face.index].uv)
                    uv_im.append(uvtex.data[face.index].image.name)
                    uv_tex_flag=1

        #collect Vertex color if any (only selected one is used
        if len(me.vertex_colors)>0:
            mesh_color=me.active_vertex_color.data[face.index]
            vert_col.append([mesh_color.color1[0],mesh_color.color1[1],mesh_color.color1[2]])
            vert_col.append([mesh_color.color2[0],mesh_color.color2[1],mesh_color.color2[2]])
            vert_col.append([mesh_color.color3[0],mesh_color.color3[1],mesh_color.color3[2]])
            vert_col_flag=1

        for i in range(3):
            vert_index=face.verts[i]

            vxyz[i]=me.verts[vert_index].co

            if (face.smooth):
                vnxyz[i]=me.verts[vert_index].normal
            else:
                vnxyz[i]=face.normal

        #Writing results to file
        for i in range(3):
            for j in range(3):
                # Write Vertices coords.
                file.write ("%+06f " % vxyz[i][j])
            file.write(" ")
            for j in range(3):
                # Write vertices normals
                file.write ("%+06f " % vnxyz[i][j])
            file.write(" ")

            #Writing uv texture information
            if (uv_tex_flag):
                file.write("%06f %06f %s" % (uv[0][i][0],uv[0][i][1],uv_im[0]))
                #Handle multi texturing
                for j in range (1,len(uv_im)):
                    file.write(";%06f|%06f|%s" % (uv[j][i][0],uv[j][i][1],uv_im[j]))
            # No texturing information, use vertex color
            elif (vert_col_flag):
                    file.write("0 0 rgb(%4f,%4f,%4f)" % (vert_col[i][0],vert_col[i][1],vert_col[i][2]))
            else: #No vertex color using gray color.
                    file.write("0 0 rgb(%3.3f,%3.3f,%3.3f)" % (0.6,0.6,0.6))

            file.write("\n")
            li=li+1

            if (li%100==0):
                print (li)


def export_tri(filen):

    print("\n\n\n\n\n\n\n\n")

    if (bpy.data.filename[0]=='<'):
        file=create_file(filen)
    else:
        file=create_file(bpy.data.filename)

    save_context()

    ob=bpy.context.active_object
    obn=create_working_object(ob)
    me=obn.data
    export(me,file)

    delete_working_object(obn)

    file.flush()
    file.close()
    restore_context()

    return



class ExportTri(bpy.types.Operator):
    '''Export to Raydium .tri file.'''
    bl_idname = "export.tri"
    bl_label = "Export Raydium Tri Format"

    # List of operator properties, the attributes will be assigned
    # to the class instance from the operator settings before calling.


    path = StringProperty(name="File Path", description="File path used for exporting the Raydium tri file", maxlen= 1024, default= "test.tri")
    use_modifiers = BoolProperty(name="Apply Modifiers", description="Apply Modifiers to the exported mesh", default= True)
    use_normals = BoolProperty(name="Normals", description="Export Normals for smooth and hard shaded faces", default= True)
    use_uvs = BoolProperty(name="UVs", description="Exort the active UV layer", default= True)
    use_colors = BoolProperty(name="Vertex Colors", description="Exort the active vertex color layer", default= True)


    def poll(self, context):
        return context.active_object != None

    def execute(self, context):
        # print("Selected: " + context.active_object.name)

        if not self.properties.path:
            raise Exception("filename not set")
        export_tri(self.properties.path)

        return ('FINISHED',)

    def invoke(self, context, event):
        wm = context.manager
        wm.add_fileselect(self)
        return ('RUNNING_MODAL',)

    def draw(self, context):
        layout = self.layout
        props = self.properties
        row = layout.row()
        row.prop(props, "use_modifiers")
        row.prop(props, "use_normals")
        row = layout.row()
        row.prop(props, "use_uvs")
        row.prop(props, "use_colors")


bpy.ops.add(ExportTri)

import dynamic_menu

def menu_func(self, context):
    default_path = bpy.data.filename.replace(".blend", ".tri")
    self.layout.operator(ExportTri.bl_idname, text="Raydium (.tri)...").path = default_path

menu_item = dynamic_menu.add(bpy.types.INFO_MT_file_export, menu_func)

if __name__ == "__main__":
    bpy.ops.export.tri(path="test.tri")
    #export_tri()